extensions [
  array 
  table 
  goo]


globals[ 
;;Related to the basic model of PSO
c1 ;PSO parameter for personal best
c2 ;PSO parameter for social best
w  ;PSO parameter of inertia
vmax ; upper bound for the coordinates defined in the model (limits the velocity)


;;Related to minimization
true-best-position ;position of the minimum of the function
minimum ;minimum value of the function until current step
maximum  ;maximum value of the function until current step
global-best-position ;the position with best value remembered by the whole population
global-best-value ;best value remembered by the whole population
global-changes  ;number of changes of the global-best-position and global-best-value

;; Related to neighborhoods
degree  ;mean degree (number of neighbors)
complexity  ;number of neighbors

maxR ; maximun ratio
vector-template 

;;User Information
information
text
User-guide
goal_info

;; Related to the diferent modesls of pso
rand
T
w_start
w_end
mod-inicial ; (Básico, Clerc o Trelea)
 ]


;;  -------------------
turtles-own
[ current-position
  velocity
 personal-best-val 
 personal-best-position
 local-best-val 
 local-best-position
 current-val
 step-length
 trajectory
 num-of-neighbors
 distance-to-true-best
]



;;  -----------------------------------------------------------------
;;  -----------------------------------------------------------------

;;  -------------------  MODELS MENU -----------------------
to reset
;; clear the model and set default configuration  
clear-all
reset-ticks
clear-links
;;default menu parameters:
set function "f_0"
set the-Model "STATIC-INERTIA"
set agent-shape "Default"
set agent-size 1
goo:hide "R"
set User-guide "Choose and option of 'the-Model' and then press '1:load-model'"
set goal_info "gbest-val"
goo:hide "goal_info" 
;;default execution parameters:
benchmark-functions-default
set P 20 
set Iterations 10000
set setGoal->iterations false
set concurrent true
set with-R false
;;default other parameters:
goo:show "w"
set T 0
set w "?"
set vmax "?"
set c1 "?"
set c2 "?"



end

to load-model  
showInfo
set User-guide "Configure the remaining parameters below and click 'Setup'"
goo:hide "R"
ifelse (the-model = "WITHOUT-INERTIA")
[goo:hide "w" ]
[goo:show "w" ]
initialize-intertia
ini-c1-c2
set Iterations 10000
end


;;;;  ---- Information panel
to showInfo
    
if (the-Model ="WITHOUT-INERTIA") 
[set information  "WITHOUT-INERTIA: \n Se trata del modelo básico de PSO, donde no figura el término inercia. "]
  
if (the-Model ="RANDIW") 
[set information  "PSO-RANDIW: \n Este modelo propone modificar de forma aleatoria el valor del peso inercial en entornos dinámicos. "]

if (the-Model ="LDIW") 
[set information "Linear Decreasing Inertia Weight PSO: \n En este modelo el valor del peso inercial disminuye de forma lineal"]

if (the-Model ="STATIC-INERTIA") 
[set information "Se conoce como Static-inertia PSO aquel algoritmo básico de PSO que incluye el termino inercia como una constante. "]
    
if (the-Model ="PSO-G") 
[set information  "PSO-G:\n Las partículas son atraídas hacia la mejor posición conocida (G) por el enjambre. "]
  
if (the-Model ="PSO-VG") 
[set information  "PSO-VG:\n Esta es la simplificación que tan solo tiene la atracción hacia la mejor posición (G) conocida por el enjambre. "]

if (the-Model ="PSO-PG") 
[set information "PSO-PG:\n Esta simplificación no tiene velocidad recurrente, tansolo la atracción de las partículas hacia su propia mejor posición conocida (P). "]


end
;;  ------------------- end   MODELS MENU  -----------------------




;;;;  ------------------- VISUALIZAR INDIVIDUO  -----------------------
to visualize
ask turtles
[set size agent-size set shape agent-shape]
end
;;;;  ------------------- end  VISUALIZAR INDIVIDUO  -----------------------





;;  ------------------- INERTIA (models: default,WITHOUT-INERTIA,STATIC-INERTIA,RANDIW,LDIW)-----------------------
to initialize-intertia
  set w 0.729 ;; by default
  if (the-Model ="WITHOUT-INERTIA") 
  [set w 0]
  if (the-Model ="STATIC-INERTIA")
    [
    ;making questions to the user:
    let respuesta user-one-of "Desea introducir su propio peso inercial?" ["SI" "NO"]
    ifelse (respuesta = "SI")
    [set w read-from-string user-input "Introduzca el peso inercial que desea:"
     set mod-inicial  ""
    ]
    [
      set mod-inicial user-one-of "Seleccione un modelo para definir el peso inercial" ["Básico" "Clerc" "Trelea"]
      if(mod-inicial = "Básico") [set w 0.9]
      if(mod-inicial = "Clerc") [set w 0.729]
      if(mod-inicial = "Trelea") [set w 0.6]
    ]
      
    ]
  if (the-Model ="RANDIW") 
    [
    set rand random-float 2  
    while [rand > 1] [set rand random-float 2]
    set w (0.5 + (rand / 2))
    ]
  if (the-Model ="LDIW") 
    [
    set T 0
    set w_start 1.4
    set w_end 0
    set w w_end + ((w_start - w_end) * (1 - (T / Iterations)))
    ]
end

to update-inertia
 
  if (the-Model ="RANDIW") 
    [ set rand random-float 2  
    while [rand > 1] [set rand random-float 2]
    set w (0.5 + (rand / 2))
    ]
  if (the-Model ="LDIW") [set w w_end + ((w_start - w_end) * (1 - (T / Iterations)))]
end
;;  ------------------- end  INERTIA -----------------------




;;  ------------------- SOCIAL AND PERSONAL COMPONENT -----------------------
to ini-c1-c2
 set c1 1.494
 set c2 1.494
 set vmax max-pxcor
 if (the-Model = "STATIC-INERTIA")
  [
     ;making questions to the user:
     let respuesta user-one-of "Desea introducir sus propios coeficientes c1 y c2?" ["SI" "NO"]
     ifelse (respuesta = "SI")
     [set w read-from-string user-input "Introduzca el peso inercial que desea:"
      set mod-inicial  ""
     ]
     [
      set mod-inicial user-one-of "Seleccione un modelo para definir los coeficientes" ["Básico" "Clerc" "Trelea"]
      if(mod-inicial = "Básico") [set c1 2 set c2 2 set vmax max-pxcor]
      if(mod-inicial = "Clerc") [set c1 1.494 set c2 1.494  set vmax max-pxcor]
      if(mod-inicial = "Trelea") [set c1 1.7 set c2 1.7 set vmax max-pxcor]
     ]
   ]
 if (the-Model ="LDIW") 
 [set vmax 2]
end
;;  ------------------- end  SOCIAL AND PERSONAL COMPONENT -----------------------




;;  ------------------- OBJECTIVE FUNCTIONS -----------------------
to-report summatory [v]
  let s 0
  repeat length v
  [set s (first v) + s
   set v bf v] 
report s
end

to-report productory [v]
report reduce [?1 * ?2] v
end

to-report apply-function [pos]
 report runresult word function  " pos "
end

to set-vector-template
  let li [] 
  let i 1
  repeat D 
  [set li lput i li
    set i i + 1]
  set vector-template li
end

to-report f_0 [pos]
 report summatory( map [? ^ 2] pos)
end

to-report f_1 [pos]
 report summatory( map [(100 * (?2 - (?1 ^ 2)) ^ 2) + ((?1 - 1) ^ 2)] (bf pos) (bl pos) )
end

to-report f_2 [pos]
 report summatory map [(? ^ 2) - (10 * ( cos (2 * pi * ?))) + 10] pos
end

to-report f_3 [pos]
 report ((1 / 4000) * (summatory (map [? ^ 2 ] pos)))  - (productory (map[(cos (?1 / sqrt (?2)))] pos vector-template))+ 1
end

to-report f_6 [pos]
  let s summatory (map [? ^ 2] pos)
 report 0.5 +(( (sin (sqrt s))  - 0.5) / (1.0 + (0.001 * s)))
end


;;  ------------------- end  OBJECTIVE FUNCTIONS -----------------------




;;  ------------------- INITIALIZE BIRDS RANDOMLY -----------------------
to set-random-position
  set current-position []
  repeat D 
 [set current-position lput  ((random-float max-pxcor) - (random-float max-pxcor)) current-position]
end

to set-random-velocity
  set velocity []
  repeat D 
 [set velocity lput  ((random-float max-pxcor) - (random-float max-pxcor)) velocity]
end


to set-maximum
 let the-worst max [current-val] of turtles
 if maximum < the-worst [set maximum the-worst]
end
to set-minimum
 let the-best min [current-val] of turtles
 if minimum < the-best [set minimum the-best]
end

to initialize-turtles
 create-turtles P
 ask turtles [
 set-random-position
 set-random-velocity
 set current-val apply-function current-position
 set personal-best-val current-val
 set personal-best-position current-position
 set trajectory 0
 set xcor item 0 current-position
 ifelse D > 1 [set ycor item 1 current-position] [set ycor 0]]
end
;;  -------------------fin INITIALIZE BIRDS RANDOMLY----------------------

;;  -------------------PBEST, LBEST & GBEST-----------------------
to update-personal
 if (current-val < personal-best-val )
 [
 set personal-best-position current-position
 set personal-best-val current-val
 ]
end

to initialize-global
 let the-best min-one-of turtles [personal-best-val]
 set global-best-position [personal-best-position] of the-best
 set global-best-value [personal-best-val] of the-best
 set global-changes 0 
 ask turtles [set local-best-position global-best-position 
   set local-best-val global-best-value]
end


to update-global
  ifelse (ticks = 0)
  [
 
    let the-best min-one-of turtles [personal-best-val]
    set global-best-position [personal-best-position] of the-best
    set global-best-value [personal-best-val] of the-best
    set global-changes 0
    ask turtles [set local-best-position global-best-position 
    set local-best-val global-best-value]
    
  ]
  [ 
     ask turtles [if (personal-best-val  < global-best-value )
     [set global-best-position  personal-best-position 
     set global-best-value  personal-best-val
     set global-changes global-changes + 1]]
     ifelse with-R = false 
     [ask turtles [set local-best-position global-best-position 
     set local-best-val global-best-value]]
     [ask turtles [update-neighbors]]     
  ]
end





;;  -------------------end PBEST & GBEST-----------------------




;;  -------------------RATIO-----------------------??
to-report turtles-in-D-radius
  report  turtles with [(euclidean-distance current-position ([current-position] of myself)) <= R]
end
;;  -------------------end  RATIO-----------------------




;;  -------------------NEIGHBORS-----------------------
to update-neighbors
 let them turtles-in-D-radius
 let  the-best-neighbor min-one-of  them  [personal-best-val]
 ifelse (with-R)
  [set num-of-neighbors (count them)]
  [set num-of-neighbors (P - 1)]
 let new-local-best-val apply-function ([current-position] of the-best-neighbor) 
 if (new-local-best-val < local-best-val)
 [
 set local-best-position ([personal-best-position] of the-best-neighbor)
 set local-best-val new-local-best-val
 ]
end

to update-complexity
 set complexity complexity + sum [num-of-neighbors] of turtles
end
;;  -------------------end  NEIGHBORS-----------------------







;;  -------------------LOAD MODEL SETTINGS-----------------------


to set-goal
ifelse (setGoal->iterations)
[
   set Goal iterations
   set goal_info "iterations"
      goo:show "goal_info"
]
[ goo:hide "goal_info"]
end  

to  benchmark-functions-default
   set goal_info "gbest-val"
   goo:hide "goal_info" 
  if (function = "f_0")
  [
   set D 30
   set Goal 0.01
  ]
    if (function = "f_1")
  [
   set D 30
   set Goal 100
  ]

    if (function = "f_2")
  [
   set D 30
   set Goal 100
  ]
  
    if (function = "f_3")
  [
   set D 30
   set Goal 0.1
  ]
  
    if (function = "f_6")
  [
   set D 2
   set Goal  0.00010
  ]
end


to setup-default-val
 ;;clear execution and let start runing the model again with the same or different experiment settings
 clear-ticks
 reset-ticks
 clear-turtles
 clear-patches
 clear-drawing
 clear-all-plots
 clear-output
 random-seed new-seed 
 set T 0
 set mod-inicial  ""
 set User-guide "Run the model using 'run' (use '1 step' to go step by step)"
  ;;initialize some parameters of the model
 set with-R false
 goo:hide "R"
 set Concurrent true
 set setGoal->iterations false
 set Iterations 10000
 set-goal
 set maxR sqrt (D *( max-pxcor ^ 2))
 set-vector-template
 benchmark-functions-default
 initialize-turtles
 visualize
 ask turtles [update-neighbors]
 set maximum max [current-val] of turtles
 set minimum min [current-val] of turtles 
 initialize-global
 set complexity 0
 update-inertia
end

to setup-own-val
 ;;clear execution and let start runing the model again with the same or different experiment settings
 clear-ticks
 reset-ticks
 clear-turtles
 clear-patches
 clear-drawing
 clear-all-plots
 clear-output
 random-seed new-seed 
 set T 0
 set User-guide "Run the model using 'run' (use '1 step' to go step by step)"
 ifelse with-R 
 [goo:show "R"]
 [goo:hide "R"]
 ;;initialize some parameters of the model
 set-goal
 set maxR sqrt (D *( max-pxcor ^ 2))
 set-vector-template
 initialize-turtles
 visualize
 ask turtles [update-neighbors]
 set maximum max [current-val] of turtles
 set minimum min [current-val] of turtles 
 initialize-global
 set complexity 0
 update-inertia
end
;;  -------------------end LOAD MODEL SETTINGS-----------------------




;;  -------------------RUN-----------------------

to-report goal-reached
  ifelse (setGoal->iterations)
  [
    ifelse (T >= Goal)
    [report true]
    [report false]
  ]
  [
    ifelse (( global-best-value <= Goal) or (T >= Iterations))
    [report true]
    [report false]
  ]
  
end

to go
 set User-guide "press 'setup' to run again the model or 'reset' to run a new model"
 if goal-reached [stop]
 ifelse Concurrent = 1 [fly][fly-non-concurrent]
 update-global
 set-maximum
 set-minimum
 update-complexity
 set degree(mean [num-of-neighbors] of turtles)
 set-current-plot "Mean_Degree"
 plot degree
 tick
 set T (T + 1)
 update-inertia
end

;;  ------------------- end RUN-----------------------





;;  ------------------- VEL AND POS UPDATING  (models: default,PSO-VG,PSO-G,PSO-PG)-----------------------
to-report calculate-new-velocity [v]
  
 if ((the-Model = "WITHOUT-INERTIA") or (the-Model = "STATIC-INERTIA") or (the-Model = "RANDIW") or (the-Model = "LDIW")) ;;default
  [let aux-v v
  let i 0
  repeat D
  [ let vi item i v
   let pbpi item i personal-best-position
   let nbpi item i local-best-position
   let xi item i current-position
   set vi ((w * vi) +  (c1 * random-float(1.0) * (pbpi - xi) ) +  (c2 * random-float(1.0) * (nbpi - xi)))
   if (abs vi) > vmax [set vi (sign vi) * vmax]
   set aux-v replace-item  i aux-v vi
   set i i + 1]
  report aux-v]
  
 if (the-Model = "PSO-G") 
 [  let aux-v v
    let i 0
    repeat D
  [ let nbpi item i local-best-position
    let xi item i current-position
    let vi (c2 * random-float(1.0) * (nbpi - xi))
    if (abs vi) > vmax [set vi (sign vi) * vmax]
    set aux-v replace-item  i aux-v vi
    set i i + 1
  ]
   report aux-v
 ]
 if (the-Model = "PSO-VG") 
 [
  let aux-v v
    let i 0
    repeat D
  [ let vi item i v
    let nbpi item i local-best-position
    let xi item i current-position
    set vi ((w * vi) + (c2 * random-float(1.0) * (nbpi - xi)))
    if (abs vi) > vmax [set vi (sign vi) * vmax]
    set aux-v replace-item  i aux-v vi
    set i i + 1
  ]
   report aux-v
   
 ]    
 if (the-Model = "PSO-PG") 
 [
  let aux-v v
    let i 0
    repeat D
  [ let pbpi item i personal-best-position
    let nbpi item i local-best-position
    let xi item i current-position
    let vi ((c1 * random-float(1.0) * (pbpi - xi) ) + (c2 * random-float(1.0) * (nbpi - xi)))
    if (abs vi) > vmax [set vi (sign vi) * vmax]
    set aux-v replace-item  i aux-v vi
    set i i + 1
  ]
   report aux-v  
 ]
    
end

to-report calculate-new-position [x v]
   let aux-x x
  let i 0
  repeat D
 [ let xi item i x
   let vi item i v
   set aux-x replace-item  i  aux-x (xi + vi)
    set i i + 1] 
 report aux-x
end
;;  ------------------- end VEL AND POS UPDATING-----------------------





;;  ------------------- AUX OPERATIONS-----------------------
to-report module [v]
  ifelse D = 1 [report abs first v]
  [report sqrt(reduce [?1 + ?2] (map [? ^ 2] v))]
end

to-report euclidean-distance [v1 v2]
  let aux (map [(?1 - ?2)] v1 v2)
  report module aux
end

to-report sign [num]
  ifelse num < 0 [report -1] [report 1]
end
;;  ------------------- end AUX OPERATIONS-----------------------






;;  ------------------- BIRDS FLYGING-----------------------
to fly
 ask-concurrent turtles[
 set velocity calculate-new-velocity velocity
 let new-position calculate-new-position current-position velocity
  set distance-to-true-best euclidean-distance current-position new-position
  set current-position new-position
  set current-val apply-function current-position
  set step-length module velocity
  set trajectory trajectory + step-length
  update-personal
let x item 0 current-position
let y 0
if D > 1 [set y item 1 current-position]
facexy x y
forward module list x y
 ]
end
 
to fly-non-concurrent
 ask turtles[
 set velocity calculate-new-velocity velocity
 let new-position calculate-new-position current-position velocity
  set distance-to-true-best euclidean-distance current-position new-position
  set current-position new-position
  set current-val apply-function current-position
  set step-length module velocity
  set trajectory trajectory + step-length
  update-personal
let x item 0 current-position
let y 0
if D > 1 [set y item 1 current-position]
facexy x y
forward module list x y
 ]
end
;;  ------------------- end BIRDS FLYGING-----------------------


 
@#$#@#$#@
GRAPHICS-WINDOW
349
58
983
713
150
150
2.0731
1
10
1
1
1
0
0
0
1
-150
150
-150
150
0
0
1
ticks
30.0

CHOOSER
1049
535
1141
580
agent-shape
agent-shape
"Default" "bird side"
0

CHOOSER
1154
535
1246
580
agent-size
agent-size
1 3 5
0

BUTTON
40
450
165
486
SETUP-default-values
setup-default-val\n\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

CHOOSER
181
180
319
225
function
function
"f_0" "f_1" "f_2" "f_3" "f_6"
0

MONITOR
1001
235
1071
280
w
w
6
1
11

MONITOR
1081
234
1153
279
c1
c1
17
1
11

MONITOR
1082
288
1153
333
NIL
c2
17
1
11

CHOOSER
16
45
179
90
the-Model
the-Model
"STATIC-INERTIA" "WITHOUT-INERTIA" "RANDIW" "LDIW" "PSO-G" "PSO-VG" "PSO-PG"
2

MONITOR
1005
398
1143
439
NIL
Minimum
5
1
10

SLIDER
173
340
316
373
R
R
0
1095
145
1
1
NIL
HORIZONTAL

MONITOR
1244
290
1294
335
P
P
17
1
11

MONITOR
1174
235
1294
280
iterations-max
iterations
17
1
11

BUTTON
71
532
134
565
run
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
1002
288
1072
333
NIL
vmax
17
1
11

MONITOR
1147
449
1281
490
NIL
global-best-value
15
1
10

MONITOR
1146
398
1284
439
Maximum
maximum
5
1
10

MONITOR
1006
449
1143
490
global-change-tick
global-changes
17
1
10

INPUTBOX
11
382
124
442
Goal
0.01
1
0
Number

INPUTBOX
11
180
124
240
P
20
1
0
Number

INPUTBOX
11
247
124
307
D
30
1
0
Number

SWITCH
209
307
316
340
with-R
with-R
1
1
-1000

PLOT
1029
67
1262
190
Mean_Degree
ticks
degree
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" ""

SWITCH
195
250
318
283
Concurrent
Concurrent
1
1
-1000

BUTTON
179
533
249
566
1 step
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
348
10
1296
55
Model Information:
Information
17
1
11

BUTTON
113
582
200
615
- reset  -
reset
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
199
52
317
85
load-model
load-model
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
1121
587
1183
620
NIL
visualize
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
10
112
328
149
User-guide:
User-guide
17
1
9

TEXTBOX
1045
204
1165
232
Basic flight parameters:
11
0.0
1

TEXTBOX
1158
218
1173
243
|
20
0.0
1

TEXTBOX
1158
250
1173
275
|
20
0.0
1

TEXTBOX
1195
204
1271
232
Execution parameters:
11
0.0
1

TEXTBOX
1158
283
1173
308
|
20
0.0
1

TEXTBOX
1158
315
1173
345
|
20
0.0
1

TEXTBOX
1117
371
1180
389
Results:
14
12.0
1

TEXTBOX
1005
497
1287
522
-----------------------
20
0.0
1

TEXTBOX
1109
515
1199
533
Change view:
11
0.0
1

TEXTBOX
130
10
210
28
Choose Model:
11
0.0
1

TEXTBOX
10
79
323
108
________________________
25
102.0
1

TEXTBOX
11
10
348
40
________________________
25
102.0
1

TEXTBOX
97
160
247
178
Experiment Settings:
11
0.0
1

TEXTBOX
13
477
326
509
________________________
25
102.0
1

TEXTBOX
114
512
205
530
Run experiment:
11
0.0
1

TEXTBOX
1006
346
1284
368
-----------------------
20
0.0
1

MONITOR
65
402
124
439
NIL
goal_info
17
1
9

MONITOR
1174
290
1239
335
current It.
T
17
1
11

SWITCH
162
395
316
428
setGoal->iterations
setGoal->iterations
1
1
-1000

INPUTBOX
11
315
124
375
Iterations
10000
1
0
Number

BUTTON
173
450
299
486
SETUP-own-values
setup-own-val
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

@#$#@#$#@
## WHAT IS IT?

The model implements some Particle Swarm Optimization (PSO) algorithms.

PSO is a population based stochastic optimization technique developed by Dr. Eberhart and Dr. Kennedy in 1995, inspired by social behavior of birds flying. The system is initialized with a population of potential solutions, called particles, who fly through the problem space by following the current optimum particles. 

In this NetLogo model, the turtles simulate the fly of that population of particles.



## HOW IT WORKS

The basic PSO model is the one called "STATIC-INERTIA". Also some variants of the PSO basic model are provided. Then, in this NetLogo model you can see how different models of PSO works. This variants of PSO make changes in the initialization and updating of the 'inertia weight (w)', other fly parameters like 'c1'and 'c2' and also in the velocity and position equations.



## HOW TO USE IT

1- Press Reset
2- Select a model in 'the model'
3- Press 'load-model'
4- Choose an objetive function in 'function'
5- Press 'SETUP-default-values'
6- (optional) Change settings of 'Experiment Settings' and then press 'SETUP-own-values'
7- Run the model with 'run'(all iterations) or with '1 step'(one iteration)
8- See results in section 'Results'

Before that, you can make another experiment:
-The same PSO model, the same function --> repeat from step 5(to change experiment settings) or repeat from step 7(not to change experiment)
-The same PSO model, another function --> repeat from step 4
-Another PSO model --> repeat from step 1



## EXTENDING THE MODEL

Using the provided basic model of PSO, you can implement new different PSO models as we have made in 'the-Model' and expand the application.



## CREDITS AND REFERENCES
Based on the algorithms presented in the following papers:

- C. Trelea, The particle swarm optimization algorithm: Convergence analysis and parameter selection, Information Processing Letters 85 (6) (2003) 317–325.
- Eberhart, R.C.; Shi, Y., "Comparing inertia weights and constriction factors in particle swarm optimization," Evolutionary Computation, 2000. Proceedings of the 2000 Congress on , vol.1, no., pp.84,88 vol.1, 2000
doi: 10.1109/CEC.2000.870279
- Huang, C., Zhang, Y., Jiang, D., Xu, B. On some non-linear decreasing inertia weight strategies in particle swarm optimization .(2007) Proceedings of the 26th Chinese Control Conference, CCC 2007, art. no. 4347175, pp. 750-753.
- Y. Shi, R.C. Eberchart, A modified particle swarm optimizer, in: Proceedings of IEEE International Conference on Evolutionary Computation, Anchorage, AK, 1998, pp.69-73.
- A. Nickabadi, M.M. Ebadzadeh, R. Safabakhsh, A novel particle swarm optimization algorithm with adaptive inertia weight, Applied Soft Computing 11(4) (2011), pp. 3658-3670.
- Pedersen, Magnus Erik Hvass. Tuning & simplifying heuristical optimization. Diss. PhD thesis, University of Southampton, 2010
- C. Trelea, The particle swarm optimization algorithm: Convergence analysis and parameter selection, Information Processing Letters 85 (6) (2003) 317–325.

## HOW TO CITE

If you mention this model in a publication, we ask that you include these citations for the model itself and for the NetLogo software:

- Cases, Blanca Rosa; Gaztelu, Maider and Moujahid, Abdelmalik (2013). NetLogo Particle Swarm Optimization 	teaching models. Computer Engineering Faculty, University of the Basque Country, Basque Country, Spain.
- Wilensky, U. (1999). NetLogo. http://ccl.northwestern.edu/netlogo/. Center for Connected Learning and Computer-Based Modeling, Northwestern Institute on Complex Systems, Northwestern University, Evanston, IL.

## COPYRIGHT AND LICENSE

<a rel="license" href="http://creativecommons.org/licenses/by-nc-sa/3.0/"><img alt="Creative Commons License" style="border-width:0" src="http://i.creativecommons.org/l/by-nc-sa/3.0/88x31.png" /></a><br />[CC BY-NC-SA 3.0] 2013 Blanca Rosa Cases, Maider Gaztelu and Abdelmalik Moujahid, University of the Basque Country


This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License. To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/3.0/.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bird side
false
0
Polygon -7500403 true true 0 120 45 90 75 90 105 120 150 120 240 135 285 120 285 135 300 150 240 150 195 165 255 195 210 195 150 210 90 195 60 180 45 135
Circle -16777216 true false 38 98 14

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="1dimensional" repetitions="100" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="10000"/>
    <metric>global-best-value</metric>
    <enumeratedValueSet variable="function">
      <value value="&quot;f_0&quot;"/>
      <value value="&quot;f_2&quot;"/>
      <value value="&quot;f_3&quot;"/>
      <value value="&quot;f_6&quot;"/>
    </enumeratedValueSet>
    <steppedValueSet variable="P" first="1" step="1" last="30"/>
    <enumeratedValueSet variable="R">
      <value value="9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Goal">
      <value value="0.1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="D">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Concurrent">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="with-R">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Model">
      <value value="&quot;set-Basic-mode&quot;"/>
      <value value="&quot;set-Clerc-mode&quot;"/>
      <value value="&quot;set-Trelea-mode&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="1dimensional-concurrent" repetitions="100" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="10000"/>
    <metric>global-best-value</metric>
    <enumeratedValueSet variable="function">
      <value value="&quot;f_0&quot;"/>
      <value value="&quot;f_2&quot;"/>
      <value value="&quot;f_3&quot;"/>
      <value value="&quot;f_6&quot;"/>
    </enumeratedValueSet>
    <steppedValueSet variable="P" first="1" step="1" last="30"/>
    <enumeratedValueSet variable="R">
      <value value="9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Goal">
      <value value="0.1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="D">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Concurrent">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="with-R">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Model">
      <value value="&quot;set-Basic-mode&quot;"/>
      <value value="&quot;set-Clerc-mode&quot;"/>
      <value value="&quot;set-Trelea-mode&quot;"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
